/************************************************************************/
/*                                                                      */
/*  This example passes data between Microsoft Excel and LabWindows/CVI */
/*  using DDE.  You need Microsoft Excel to run this example.           */
/*                                                                      */
/************************************************************************/

#include <cvirte.h>    /* Needed if linking in external compiler; harmless otherwise */
#include <ddesupp.h>
#include <utility.h>
#include <userint.h>
#include <ansi_c.h>
#include <analysis.h>
#include "ddedemo.h"

void GenerateWave (int shape);

double datapoints[100];

int CVICALLBACK ClientCallback(unsigned int dde_handle, char *topicName, char *itemName,
            int event, int dataFmt, int dataSize, void *dataPtr, void *callbackData);

int status=0;
unsigned datalink, systemlink;
int panelhandle;
int quit=0;
static char proj_dir[MAX_PATHNAME_LEN];
static char filename[MAX_PATHNAME_LEN];

int main (int argc, char *argv[])
{
    int     dde_handle, i;

    if (InitCVIRTE (0, argv, 0) == 0)    /* Needed if linking in external compiler; harmless otherwise */
        return -1;    /* out of memory */

    panelhandle = LoadPanel (0, "ddedemo.uir", PANEL);
    SetPanelPos (panelhandle, 30, VAL_AUTO_CENTER);
    DisplayPanel (panelhandle);
    RunUserInterface ();
    
    return 0;
}


int CVICALLBACK LaunchExcel(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    char path [MAX_PATHNAME_LEN];
    char projDir [MAX_PATHNAME_LEN];

    if (event == EVENT_COMMIT) {
        SetInputMode (panelhandle, PANEL_LAUNCH, 0);
        GetCtrlVal(panelhandle, PANEL_PATH, path);
        GetProjectDir (projDir);

        sprintf (path, "%s %s\\%s", path, projDir, "lwcvi.xls");
        LaunchExecutable (path);
        SetInputMode (panelhandle, PANEL_CONNECT, 1);
    }
    return (0);
}



/*************************************************************************/
/*                                                                       */
/*  The ConnectToExcel function performs three operations:               */
/*                                                                       */
/*      1.  It connects to the Excel "system", to make sure Excel is     */
/*          running and can be accessed.  If you want to send commands   */
/*          to the Excel application, like Open, Save, etc, you would    */
/*          first connect to the Excel system.                           */
/*                                                                       */
/*      2.  Next, it connects to the worksheet entitled "LWCVI.XLS". If  */
/*          you want to pass data to a worksheet, you only need to       */
/*          connect to that worksheet                                    */
/*                                                                       */
/*      3.  Finally, this function establishes a warm link between the B5*/
/*          cell in the Excel worksheet and LabWindows/CVI.  Therefore,  */
/*          whenever the value in B5 changes, Excel will automatically   */
/*          send the new value to LabWindows/CVI through a callback      */
/*          function.                                                    */
/*                                                                       */
/*************************************************************************/

int CVICALLBACK ConnectToExcel(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    char valStr[100];
    double value;
    
    if (event == EVENT_COMMIT){
        SetCtrlVal (panelhandle, PANEL_STATUS, "Connecting to Excel...\n");
        GetProjectDir (proj_dir);
        sprintf (filename, "%s\\lwcvi.xls", proj_dir);
        status = ConnectToDDEServer(&datalink, "excel", filename, ClientCallback, 0);
        status = ConnectToDDEServer(&systemlink, "excel", "system", ClientCallback, 0);
        if (status<0) {
            SetCtrlVal (panelhandle, PANEL_STATUS, "Connection to Work Sheet failed.\n");
            }
        else {
            SetCtrlVal (panelhandle, PANEL_STATUS, "Connection to Worksheet Successful\n");
            SetCtrlVal(panelhandle, PANEL_STATUS, "Connection to Excel Succesful.\n");
            SetInputMode (panelhandle, PANEL_CONNECT, 0);
            SetInputMode (panelhandle, PANEL_SEND, 1);
            }
        if ((status=SetUpDDEHotLink(datalink, "R5C2", CF_TEXT, 5000))<0)
            printf ("SetUpWarmLink Failed, %d\n", status);
        ClientDDERead (0, "R5C2", CF_TEXT, valStr, sizeof(valStr), 0);
        sscanf ((char*)valStr, "%lf", &value);
        SetCtrlVal (panelhandle, PANEL_CELLVALUE, value);
        
    }
    return (0);
}

/***********************************************************************/
/*                                                                     */
/*  The SendDataToExcel function simply passes the waveform data to    */
/*  Excel using the ClientDDEWrite function.                           */
/*                                                                     */
/***********************************************************************/

void SendDataToExcel(void)
{
    int         i, j=0;
    char        buf[2000], msg[100];

    SetCtrlVal (panelhandle, PANEL_STATUS, "Sending Data to Worksheet\n");
    for (i=0; i<50; i++) {
            sprintf(&buf[j],"%f\n", datapoints[i]);
            j=strlen(buf);
        }
    status = ClientDDEWrite(datalink, "R1C2:R50C2", CF_TEXT, buf,
        strlen(buf), 1000);

    SetCtrlVal (panelhandle, PANEL_STATUS, "Data Sent.\n");
}



/************************************************************************/
/*                                                                      */
/*  The ClientCallback function is the function that gets invoked when  */
/*  Excel sends a DDE "event" back to LabWindows/CVI.  The event can    */
/*  either be a DATAREADY event (meaning the value of cell B5 has       */
/*  changed) or it can be a DISCONNECT event (meaning Excel has been    */
/*  closed.                                                             */
/*                                                                      */
/*  When the DATAREADY event is passed, this function simply displays   */
/*  the new value of B5 on the UIR.                                     */
/*                                                                      */
/************************************************************************/

int CVICALLBACK ClientCallback(
unsigned    dde_handle,
char        *topicName,
char        *itemName,
int         event,
int         dataFmt,
int         dataSize,
void        *dataPtr,
void        *callbackData)
{
    double value;

    switch (event) {
        case DDE_DATAREADY:
            sscanf ((char*)dataPtr, "%lf", &value);
            status = SetCtrlVal (panelhandle, PANEL_CELLVALUE, value);
            break;
        case DDE_DISCONNECT:
            SetCtrlVal (panelhandle, PANEL_STATUS, "Connection Closed!!!\n");
            status = DisconnectFromDDEServer (datalink);
            status = DisconnectFromDDEServer (systemlink);

            break;
    }
    return 0;
}

/*******************************************************************/
/*                                                                 */
/*  The SendData function is called by the Send Data button on the */
/*  UIR. This function generates a waveform, and then calls the    */
/*  SendDataToExcel function which passes the datapoints over DDE  */
/*  to Excel.                                                      */
/*                                                                 */
/*******************************************************************/

int CVICALLBACK SendData(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    int shape;

    switch(event) {
        case EVENT_COMMIT :
            status = GetCtrlVal (panelhandle, PANEL_WAVEFORM, &shape);
            GenerateWave (shape);
            SendDataToExcel();
            break;
        }
    return(0);
}


void GenerateWave (int shape)
{
    int i;

    switch (shape) {
        case 1 :        /* Sinewave */
            for (i=0;i<50;i++)
                datapoints[i] = 50 * sin((((2*3.1415926539)/50) * i)) + 50;
            break;
        case 2 :        /* Square wave */
            for (i=0;i<16;i++)
                datapoints[i] = 33.0;
            for (i=16;i<33;i++)
                datapoints[i] = 67.0;
            for (i=33;i<50;i++)
                datapoints[i] = 33.0;
            break;
        case 3 :        /* Triangle wave */
            for (i=0;i<13;i++) {
                datapoints[i] = i*7.69;
                datapoints[i+13] = 100 - (i*7.69);
                datapoints[i+26] = i*7.69;
                datapoints[i+39] = 100 - (i*7.69);
                }
            break;
        }
}


int CVICALLBACK Shutdown(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    if (event == EVENT_COMMIT) {

        SetCtrlVal (panelhandle, PANEL_STATUS, "Closing Worksheet...\n");
        SetCtrlVal (panelhandle, PANEL_STATUS, "Quiting Excel...\n");

        DisableBreakOnLibraryErrors ();

        status = DisconnectFromDDEServer (datalink);
        status=ClientDDEExecute (systemlink, "[CLOSE(0)]", 0);
        status = ClientDDEExecute (systemlink, "[QUIT()]", 0);

        EnableBreakOnLibraryErrors ();

        QuitUserInterface (0);


        }
    return(0);
}
