/*************************************************************************/
/*  This program illustrates sending commands to Excel via a DDE         */
/*  connection. Commands illustrated here are: OPEN sheet, CLOSE sheet,  */
/*  RUN macro, and QUIT Excel.  Information on more commands can be      */
/*  found in Excel manuals.                                              */
/*************************************************************************/

#include <cvirte.h>    /* Needed if linking in external compiler; harmless otherwise */
#include <formatio.h>
#include <ddesupp.h>
#include <utility.h>
#include <userint.h>
#include <formatio.h>
#include "ddeexcel.h"

int CVICALLBACK ClientDDECallback(unsigned int dde_handle, char *topicName, char *itemName,
            int event, int dataFmt, int dataSize, void *dataPtr, void *callbackData);

int     status, status2;
int     panelhandle,helphandle;
int     quit;
unsigned int    dde_handle;
char projectpath[300], sheetpath[350];

int main (int argc, char *argv[])
{
    if (InitCVIRTE (0, argv, 0) == 0)    /* Needed if linking in external compiler; harmless otherwise */
        return -1;    /* out of memory */
    
    helphandle = LoadPanel (panelhandle, "ddeexcel.uir", HELP);
    panelhandle = LoadPanel (0, "ddeexcel.uir", PANEL);

    /*  Get project path to determine a default path for Excel Sheets.  */
    GetProjectDir (projectpath);
    Fmt (sheetpath, "%s<%s\\cvisheet.xls", projectpath);
    SetCtrlVal (panelhandle, PANEL_SHEETPATH, sheetpath);
    Fmt (sheetpath, "%s<%s\\macro1.xlm", projectpath);
    SetCtrlVal (panelhandle, PANEL_MACROPATH, sheetpath);

    DisplayPanel (panelhandle);
    RunUserInterface ();
    
    return 0;
}   /* end main  */


int CVICALLBACK LAUNCH_FUNC(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    char filename[50];

    if (event == EVENT_COMMIT)
    {
        SetInputMode (panelhandle, PANEL_LAUNCHEXCEL, 0);
        GetCtrlVal (panelhandle, PANEL_EXCELPATH, filename);
        SetCtrlVal (panelhandle, PANEL_STATUS, "Launching Excel...\n");

        status = LaunchExecutable (filename);

        if (status<0) {
            SetCtrlVal (panelhandle, PANEL_STATUS, "Launch Excel failed.\n");
        }
        else {
            SetCtrlVal (panelhandle, PANEL_STATUS, "Launch Excel Successful\n");
            SetInputMode (panelhandle, PANEL_DDECONNECT, 1);
        }
    }  /*  end if (event)  */
    return (0);
}   /*  end LAUNCH_FUNC  */


int CVICALLBACK DDECONNECT_FUNC(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{

    if (event == EVENT_COMMIT)
    {
        SetCtrlVal (panelhandle, PANEL_STATUS, "Connecting to Worksheet...\n");

        status = ConnectToDDEServer(&dde_handle, "excel", "system", ClientDDECallback, 0);

        if (status<0) {
            SetCtrlVal (panelhandle, PANEL_STATUS, "Connection to Worksheet failed.\n");
            }
        else {
            SetCtrlVal (panelhandle, PANEL_STATUS, "Connection to Worksheet Successful\n");
            SetCtrlAttribute (panelhandle, PANEL_DDECONNECT, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelhandle, PANEL_OPENSHEET, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelhandle, PANEL_QUITEXCEL, ATTR_DIMMED, 0);
            }
    }  /*  end if (event)  */
    return (0);
}   /*  end DDECONNECT_FUNC  */

/************************************************************************/
/*                                                                      */
/*  The command string to have Excel open a sheet is:                   */
/*                                                                      */
/*      [OPEN("sheet.xls")]                                             */
/*                                                                      */
/*      where sheet.xls is the sheet to open                            */
/*                                                                      */
/************************************************************************/

int CVICALLBACK OPEN_FUNC(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    char sheet[100];
    char command[100];

    if (event== EVENT_COMMIT)
    {
        SetCtrlVal (panelhandle, PANEL_STATUS, "Opening Sheets...\n");

        GetCtrlVal (panelhandle, PANEL_SHEETPATH, sheet);
        Fmt (command, "%s<[OPEN(\"%s\")]", sheet);

        status = ClientDDEExecute (dde_handle, command, 0);

        GetCtrlVal (panelhandle, PANEL_MACROPATH, sheet);
        Fmt (command, "%s<[OPEN(\"%s\")]", sheet);

        status2 = ClientDDEExecute (dde_handle, command, 0);

        if ((status < 0) || (status2 < 0)) {
            SetCtrlVal (panelhandle, PANEL_STATUS, "Open Sheets Failed\n");
        }
        else {
            SetCtrlVal (panelhandle, PANEL_STATUS, "Open Sheets Successful\n");
            SetCtrlAttribute (panelhandle, PANEL_RUN, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelhandle, PANEL_CLOSESHEET, ATTR_DIMMED, 0);
        }
    }   /*  end if (event)  */
    return(0);

}   /*  end OPEN_FUNC  */

/************************************************************************/
/*                                                                      */
/*  The command string to have Excel close the active sheet:            */
/*                                                                      */
/*      [CLOSE(X)]                                                      */
/*                                                                      */
/*      where X determines whether to save the sheet                    */
/*                 (zero means don't save, non-zero means save)         */
/*                                                                      */
/************************************************************************/
int CVICALLBACK CLOSE_FUNC(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    int save;
    char command[50];

    if (event== EVENT_COMMIT)
    {
        GetCtrlVal (panelhandle, PANEL_SAVESHEET, &save);
        SetCtrlVal (panelhandle, PANEL_STATUS, "Closing Sheet...\n");
        Fmt (command, "%s<[CLOSE(%i)]", save);

        status=ClientDDEExecute (dde_handle, command, 0);

        if (status < 0) {
            SetCtrlVal (panelhandle, PANEL_STATUS, "Close Sheet Failed\n");
        }
        else {
            SetCtrlVal (panelhandle, PANEL_STATUS, "Close Sheet Successful\n");
        }
    }   /*  end if (event)  */
    return(0);

}   /*  end CLOSE_FUNC  */

/************************************************************************/
/*                                                                      */
/*  The command string to have Excel quit is:                           */
/*                                                                      */
/*      [QUIT()]                                                        */
/*                                                                      */
/************************************************************************/
int CVICALLBACK QUITEXCEL_FUNC(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    if (event== EVENT_COMMIT)
    {
        SetCtrlVal (panelhandle, PANEL_STATUS, "Quiting Excel...\n");

        DisableBreakOnLibraryErrors ();
        status = ClientDDEExecute (dde_handle, "[QUIT()]", 0);
        EnableBreakOnLibraryErrors ();

        if (status < 0 && status != -8) {
            SetCtrlVal (panelhandle, PANEL_STATUS, "Quitting Excel Failed\n");
        }
        else {
            SetCtrlVal (panelhandle, PANEL_STATUS, "Quitting Excel Successful\n");
            SetCtrlAttribute (panelhandle, PANEL_RUN, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelhandle, PANEL_OPENSHEET, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelhandle, PANEL_CLOSESHEET, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelhandle, PANEL_QUITEXCEL, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelhandle, PANEL_LAUNCHEXCEL, ATTR_DIMMED, 0);
        }
    }   /*  end if (event)  */
    return(0);

}   /*  end CLOSE_FUNC  */

/************************************************************************/
/*                                                                      */
/*  The command string to have Excel execute a macro is the following:  */
/*                                                                      */
/*      [RUN("sheet.xls!macroname")]                                    */
/*                                                                      */
/*      where sheet.xls is the macro sheet and                          */
/*            macroname is the name of the macro                        */
/*              or the macro cell location in R1C1 notation format      */
/*                                                                      */
/************************************************************************/
int CVICALLBACK RUN_MACRO_FUNC(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    char sheet[50];
    char command[50];

    if (event== EVENT_COMMIT)
    {
        GetCtrlVal (panelhandle, PANEL_MACROSHEET, sheet);
        SetCtrlVal (panelhandle, PANEL_STATUS, "Running Macro...\n");
        Fmt (command, "%s<[RUN(\"%s\")]", sheet);

        status=ClientDDEExecute (dde_handle, command, 0);

        if (status < 0) {
            SetCtrlVal (panelhandle, PANEL_STATUS, "Close Sheet Failed\n");
        }
        else {
            SetCtrlVal (panelhandle, PANEL_STATUS, "Close Sheet Successful\n");
        }
    }   /*  end if (event)  */
    return(0);

}   /*  end RUN_MACRO_FUNC  */

int CVICALLBACK SHUTDOWN(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    if (event == EVENT_COMMIT) {
        QuitUserInterface (0);
    }   /*  end if (event)  */
    return(0);

}   /*  end SHUTDOWN  */

int CVICALLBACK HELP_FUNC(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    int returnhandle, returncontrol;
    if (event == EVENT_COMMIT) {
        InstallPopup (helphandle);
        GetUserEvent (1, &returnhandle, &returncontrol);
        RemovePopup (0);

    }   /*  end if (event)  */
    return(0);

}   /*  end HELP_FUNC  */

/************************************************************************/
/*                                                                      */
/*  The ClientCallback function is the function that gets invoked when  */
/*  Excel sends a DDE "event" back to LabWindows/CVI.  The event can    */
/*  either be a DATAREADY event or it can be a DISCONNECT event         */
/*  (meaning Excel has been closed).                                    */
/*                                                                      */
/************************************************************************/
int CVICALLBACK ClientDDECallback(
unsigned    dde_handle,
char        *topicName,
char        *itemName,
int         event,
int         dataFmt,
int         dataSize,
void        *dataPtr,
void        *callbackData)
{

    if (event == DDE_DISCONNECT) {
        SetCtrlVal (panelhandle, PANEL_STATUS, "DDE Connection Closed!!!\n");
    }
    return (0);
}
