/************************************************************************/
/*                                                                      */
/*  This example acts as a DDE client and establishes a DDE connection  */
/*  to the Windows Program Manager.  Once connected, you can create     */
/*  Windows program groups and items directly from the LabWindows/CVI   */
/*  program.  This program uses the ClientDDEExecute() function to send */
/*  commands to the Windows Program Manager.                            */
/*                                                                      */
/************************************************************************/

#include <cvirte.h>    /* Needed if linking in external compiler; harmless otherwise */
#include <userint.h>
#include <utility.h>
#include <ddesupp.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "prgmndde.h"

#define TRUE  1;
#define FALSE 0;

/* Global Variables */
int          bConnection = FALSE;
unsigned int Conv_Handle;
int          panel_handle;
void         *CallBack_Data;

/* The DDE callback function. */
int CVICALLBACK Call_Back_Dde(unsigned int handle, char *topicName, char *itemName,
                                int xType, int dataFmt, int dataSize,
                                void *dataPtr, void *callbackData);

int main (int argc, char *argv[])
{
    static char full_path[MAX_PATHNAME_LEN];

    if (InitCVIRTE (0, argv, 0) == 0)    /* Needed if linking in external compiler; harmless otherwise */
        return -1;    /* out of memory */

    GetFullPathFromProject ("prgmndde.uir", full_path);
    panel_handle = LoadPanel (0, full_path, PANEL);
    SetPanelPos (panel_handle, 30, 10);
    DisplayPanel (panel_handle);
    RunUserInterface ();
    
    return 0;
}


int CVICALLBACK Connection(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    int Dde_Err;

    if (event==EVENT_COMMIT)
    {
        Dde_Err = ConnectToDDEServer (&Conv_Handle, "PROGMAN", "PROGMAN",
                                                                Call_Back_Dde, CallBack_Data);
        if (Dde_Err==0)
        {
            bConnection = TRUE;
            SetCtrlAttribute (panel_handle, PANEL_ADD_GROUP, ATTR_DIMMED, 0);
            SetCtrlAttribute (panel_handle, PANEL_ADD_ITEM, ATTR_DIMMED, 0);
            SetCtrlAttribute (panel_handle, PANEL_DEL_GROUP, ATTR_DIMMED, 0);
            SetCtrlAttribute (panel_handle, PANEL_GROUP_NAME, ATTR_DIMMED, 0);
            SetCtrlAttribute (panel_handle, PANEL_ITEM_NAME, ATTR_DIMMED, 0);
            SetCtrlAttribute (panel_handle, PANEL_ICON_NAME, ATTR_DIMMED, 0);
            SetCtrlAttribute (panel_handle, PANEL_DELETE_NAME, ATTR_DIMMED, 0);
        }
    }
    return (0);
}


int CVICALLBACK Add_Group(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
  char *groupName;
  char *ddeCommand;
  int  nameLength;
  int  Dde_Err;

    if (event==EVENT_COMMIT)
    {
        GetTextBoxLineLength (panel_handle, PANEL_GROUP_NAME, 0, &nameLength);
        groupName = (char*)(calloc(nameLength+1, sizeof(char)));
        GetTextBoxLine (panel_handle, PANEL_GROUP_NAME, 0, groupName);
        ddeCommand = (char*)(calloc((31+2*nameLength), sizeof(char)));
        sprintf(ddeCommand,"[CreateGroup(%s)][ShowGroup(%s,1)]", groupName, groupName);
        Dde_Err = ClientDDEExecute (Conv_Handle, ddeCommand, 0);
        DisplayPanel (panel_handle);
        free (ddeCommand);
        free (groupName);
    }
    return (0);
}

int CVICALLBACK Add_Item(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    char *itemName;
    char *iconName;
    char *ddeCommand;
    int  itemNameLength, iconNameLength;
    int  Dde_Err;

    if (event==EVENT_COMMIT)
    {
      GetTextBoxLineLength (panel_handle, PANEL_ITEM_NAME, 0, &itemNameLength);
      itemName = (char*)(calloc(itemNameLength+1, sizeof(char)));
      GetTextBoxLine (panel_handle, PANEL_ITEM_NAME, 0, itemName);

      GetTextBoxLineLength (panel_handle, PANEL_ICON_NAME, 0, &iconNameLength);
      iconName = (char*)(calloc(iconNameLength+1, sizeof(char)));
      GetTextBoxLine (panel_handle, PANEL_ICON_NAME, 0, iconName);

      ddeCommand = (char*)(calloc((12+2+itemNameLength+iconNameLength), sizeof(char)));

      sprintf(ddeCommand,"[AddItem(%s, %s)]", itemName, iconName);
      Dde_Err = ClientDDEExecute (Conv_Handle, ddeCommand, 0);
      free (itemName);
      free (iconName);
    }
    return (0);
}



int CVICALLBACK Del_Group(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    char *groupName;
    char *ddeCommand;
    int  groupNameLength;
    int  response;
    int  Dde_Err;

    if (event==EVENT_COMMIT)
    {
        response = ConfirmPopup ("", "Are you sure you want to delete the group?");
        if (response == 0) return (-1);
        GetTextBoxLineLength (panel_handle, PANEL_DELETE_NAME, 0, &groupNameLength);
        groupName = (char*)(calloc(groupNameLength+1, sizeof(char)));
        GetTextBoxLine (panel_handle, PANEL_DELETE_NAME, 0, groupName);
        ddeCommand = (char*)(calloc((15+1+groupNameLength), sizeof(char)));
        sprintf(ddeCommand,"[DeleteGroup(%s)]", groupName);
        Dde_Err = ClientDDEExecute (Conv_Handle, ddeCommand, 0);
        DisplayPanel (panel_handle);
        free (groupName);
        free (ddeCommand);
    }

    return (0);
}


int CVICALLBACK Quit(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    int Dde_Err;

    if (event==EVENT_COMMIT)
    {
        if (bConnection)
            Dde_Err = DisconnectFromDDEServer (Conv_Handle);
        QuitUserInterface(0);
    }
    return(0);
}




int CVICALLBACK Call_Back_Dde(unsigned int handle, char *topicName, char *itemName,
                                int xType, int dataFmt, int dataSize,
                                void *dataPtr, void *callbackData)
{
    return (0);
}

