/****************************************************************************/
/* SERVER.H: This file contains the public communication protocols which    */
/* the server exports, so that communication with clients is possible. Also,*/
/* for the sake of convenience, prototypes and macros used by SRVCLT.C were */
/* placed here.                                                             */
/*                                                                          */
/****************************************************************************/
/*                                                                          */
/* Data and Commands this server program supports through DDE:              */
/*                                                                          */
/*  Server data accessible through DDE:                                     */
/*                                                                          */
/*    DDE Name       #define            DDE Support                         */
/*    --------       -------            -----------                         */
/*    "AllCells"     ALL_CELLS          A hot link can be established to    */
/*                                      this item. The client will then be  */
/*                                      notified with the new data when any */
/*                                      of the numerics on the server panel */
/*                                      is changed.                         */
/*                                                                          */
/*    "CELL1"        CELL1              The values of numerics 1 through 20 */
/*     ...            ...               on the server panel can be read     */
/*    "CELL20"       CELL20             from and written to using these     */
/*                                      DDE names. (NOTE: the value of each */
/*                                      item is a double (8 bytes))         */
/*                                                                          */
/*  Server commands accessible through DDE:                                 */
/*                                                                          */
/*    DDE Name       #define            DDE Support                         */
/*    --------       -------            -----------                         */
/*    "GenerateData" CMD_GENERATED_DATA The action of each command button   */
/*    "AddData"      CMD_ADD_DATA       on the server panel can be executed */
/*    "ShiftRight"   CMD_SHIFT_LEFT     by matching the DDE command listed  */
/*    "ShiftLeft"    CMD_SHIFT_RIGHT    to the left.                        */
/*    "Quit"         CMD_QUIT           .                                   */
/*                                                                          */
/*                                                                          */
/* The defined constant names for each DDE item and command exported by     */
/* this server are listed below.                                            */
/*                                                                          */
/****************************************************************************/


#ifndef SERVER_HEADER
#define SERVER_HEADER

/****************************************************************************/
/*                          Macro Definitions                               */
/****************************************************************************/

#define NAME        "Test Server"   /*  name under which server will be registered */
#define TOPIC       "DDE Test"      /*  topic of the conversation */
#define ALL_CELLS   "AllCells"      /*  data item used for hot link (advisory
                                        loop). The data consists of an array of 20
                                        doubles, being controlled by the numerics on the
                                        server's front panel */

#define NUM_CELLS   20              /*  number of data cells on the server panel    */
#define CELL        "CELL"          /*  to read/write the value of a server cell,   */
                                    /*  specify the cell with the string "CELLx"    */
                                    /*  (where x is the cell number).               */


#define CELL1   "CELL1"
#define CELL2   "CELL2"             /*  data items which the client can read    */
#define CELL3   "CELL3"             /*  and write.  Each item is the the value  */
#define CELL4   "CELL4"             /*  of one of the numerics on the server    */
#define CELL5   "CELL5"             /*  panel.  The data type of the values is  */
#define CELL6   "CELL6"             /*  double (four bytes)                     */
#define CELL7   "CELL7"
#define CELL8   "CELL8"
#define CELL9   "CELL9"
#define CELL10  "CELL10"
#define CELL11  "CELL11"
#define CELL12  "CELL12"
#define CELL13  "CELL13"
#define CELL14  "CELL14"
#define CELL15  "CELL15"
#define CELL16  "CELL16"
#define CELL17  "CELL17"
#define CELL18  "CELL18"
#define CELL19  "CELL19"
#define CELL20  "CELL20"


    /* Commands supported by the server: */
#define CMD_GENERATED_DATA  "GenerateData"
#define CMD_ADD_DATA        "AddData"
#define CMD_SHIFT_LEFT      "ShiftRight"
#define CMD_SHIFT_RIGHT     "ShiftLeft"
#define CMD_QUIT            "Quit"


#define DDE_TIMEOUT        5000     /* timeout threshold (in milliseconds) for
                                       DDE message requests which require it */

#define TRUE    1
#define FALSE   0

/****************************************************************************/
/*                          Function Prototypes                             */
/****************************************************************************/

void    ActivateServer (void);
void    DeactivateServer (void);


#endif /* SERVER_HEADER */
