/****************************************************************************/
/*                                                                          */
/* SRVCLT.PRJ: This program demonstrates how to use the DDE Library. It     */
/* runs as both a server and client, using DDE to communicate with itself,  */
/* as per the server/client protocol. In order to circumvent the unorthodox */
/* arrangement of having a single program mimicking two different programs, */
/* an attempt was made to separate the data and code of the two entities    */
/* into otherwise isolated modules: 'server.c' and 'client.c'. In addition, */
/* a user interface is given to both server and client, and they each load  */
/* a panel from 'srvclt.uir'.                                               */
/*      The usefulness of this program is dubious at best, but it has the   */
/* merit of using every function available in the DDE Library, exemplifying */
/* the manner in which they were intended to be used by a program acting as */
/* a traditional DDE server or client.                                      */
/*                                                                          */
/****************************************************************************/


/****************************************************************************/
/* SRVCLT.C: This module contains the thinnest layer of code necessary to   */
/* maintain separation between server and client. Its main functions are to */
/* initialize and terminate the program.                                    */
/****************************************************************************/


/****************************************************************************/
/*                          Include Files                                   */
/****************************************************************************/

#include <cvirte.h>    /* Needed if linking in external compiler; harmless otherwise */
#include <userint.h>    /* RunUserInterface() */
#include "server.h"     /* TRUE, FALSE, ActivateServer(), ActivateClient() */

extern void    ActivateClient (void);   /* defined in client.c (would be in client.h if a client.h header file was needed) */
extern void    DeactivateClient (void); /* defined in client.c (would be in client.h if a client.h header file was needed) */

/****************************************************************************/
/*                          Function Definitions                            */
/****************************************************************************/

int main (int argc, char *argv[])
{
    if (InitCVIRTE (0, argv, 0) == 0)    /* Needed if linking in external compiler; harmless otherwise */
        return -1;    /* out of memory */
    
    ActivateServer();
    ActivateClient();

        /*  The server or client will eventually call QuitUserInterface() to
            terminate this call */
    RunUserInterface();

    DeactivateClient();
    DeactivateServer();

    return 0;
}


